using UnityEngine;
using System.IO;
using System;
using HarmonyLib;
using System.Collections.Generic;

public class MoreTrashRedux : Mod
{
    static Dictionary<ObjectSpawnerAssetSettings, float[]> editedValues;
    static public JsonModInfo modInfo;
    static public bool DiffCheck;
    static public float GlobalDelay;
    static public float WorldDelay;
    static public bool GlobalCheck;
    static public bool WorldCheck;
    static public bool scheduleCheck;
    Harmony harmony;
    static public float multiplier
    {
        get
        {
            if (ExtraSettingsAPI_Loaded)
            {
                if (RAPI.GetLocalPlayer() == null || DiffCheck)
                    return GlobalCheck ? float.PositiveInfinity : GlobalDelay;
                return WorldCheck ? float.PositiveInfinity : WorldDelay;
            }
            return 0.5f;
        }
    }
    public void Start()
    {
        modInfo = modlistEntry.jsonmodinfo;
        editedValues = new Dictionary<ObjectSpawnerAssetSettings, float[]>();
        ModifyCurrentSettings();
        harmony = new Harmony("com.aidanamite.MoreTrashRedux");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    public void Update()
    {
        if (scheduleCheck)
        {
            getSettings();
            ModifyCurrentSettings();
            scheduleCheck = false;
        }
    }

    public void OnModUnload()
    {
        UnmodifyAll();
        Log("Mod has been unloaded!");
    }

    public static void ErrorLog(object message)
    {
        Debug.LogError("[" + modInfo.name + "]: " + message.ToString());
    }

    public static void WarningLog(object message)
    {
        Debug.LogWarning("[" + modInfo.name + "]: " + message.ToString());
    }

    public static void ModifyCurrentSettings()
    {
        var manager = ComponentManager<ObjectSpawnerManager>.Value;
        if (manager != null)
            foreach (var objectSpawner in new ObjectSpawner_RaftDirection[] { manager.plankSpawner, manager.itemSpawner })
                ModifySettings(Traverse.Create(objectSpawner).Field("currentSettings").GetValue<ObjectSpawnerAssetSettings>());
    }

    public static void ModifySettings(ObjectSpawnerAssetSettings set)
    {
        if (set == null)
            return;
        if (editedValues == null)
            return;
        if (editedValues.TryAdd(set, new float[] { set.spawnRateInterval.minValue, set.spawnRateInterval.maxValue }))
        {
            set.spawnRateInterval.minValue *= multiplier;
            set.spawnRateInterval.maxValue *= multiplier;
        }
        else
            RemodifySettings(set);
    }

    public static void RemodifySettings(ObjectSpawnerAssetSettings set)
    {
        if (editedValues.ContainsKey(set))
        {
            set.spawnRateInterval.minValue = editedValues[set][0] * multiplier;
            set.spawnRateInterval.maxValue = editedValues[set][1] * multiplier;
        }
    }

    public static void RemodifyAll()
    {
        if (editedValues == null || editedValues.Count <= 0)
            return;
        foreach (var pair in editedValues)
        {
            pair.Key.spawnRateInterval.minValue = pair.Value[0] * multiplier;
            pair.Key.spawnRateInterval.maxValue = pair.Value[1] * multiplier;
        }
        foreach (var objectSpawner in new ObjectSpawner_RaftDirection[] { ComponentManager<ObjectSpawnerManager>.Value.plankSpawner, ComponentManager<ObjectSpawnerManager>.Value.itemSpawner })
            Traverse.Create(objectSpawner).Field("spawnDelay").SetValue(Traverse.Create(objectSpawner).Field("currentSettings").GetValue<ObjectSpawnerAssetSettings>().spawnRateInterval.GetRandomValue());
    }

    public static void UnmodifyAll()
    {
        if (editedValues == null || editedValues.Count <= 0)
            return;
        foreach (var pair in editedValues)
        {
            pair.Key.spawnRateInterval.minValue = pair.Value[0];
            pair.Key.spawnRateInterval.maxValue = pair.Value[1];
        }
        editedValues.Clear();
    }

    public void getSettings()
    {
        if (!ExtraSettingsAPI_Loaded)
            return;
        DiffCheck = ExtraSettingsAPI_GetCheckboxState("Use global settings");
        GlobalCheck = ExtraSettingsAPI_GetCheckboxState("Disable Trash Spawn");
        if (float.TryParse(ExtraSettingsAPI_GetInputValue("Spawn Delay"), out GlobalDelay))
            GlobalDelay = Mathf.Max(GlobalDelay, 0.00001f);
        else
            GlobalDelay = 1;
        WorldCheck = ExtraSettingsAPI_GetCheckboxState("Disable Trash Spawn ");
        if (float.TryParse(ExtraSettingsAPI_GetInputValue("Spawn Delay "), out WorldDelay))
            WorldDelay = Mathf.Max(WorldDelay, 0.00001f);
        else
            WorldDelay = 1;
    }
    public void ExtraSettingsAPI_SettingsClose()
    {
        getSettings();
        RemodifyAll();
    }
    public void ExtraSettingsAPI_Load()
    {
        getSettings();
        RemodifyAll();
    }
    public void ExtraSettingsAPI_Unload()
    {
        RemodifyAll();
    }
    public void ExtraSettingsAPI_ButtonPress(string name)
    {
        if (name == "Clear Items")
            if (Semih_Network.IsHost)
                foreach (ObjectSpawner_RaftDirection objectSpawner in FindObjectsOfType<ObjectSpawner_RaftDirection>())
                    while (objectSpawner.spawnedObjects.Count > 0)
                        PickupObjectManager.RemovePickupItemNetwork(objectSpawner.spawnedObjects[0]);
    }

    static bool ExtraSettingsAPI_Loaded = false;
    public static bool ExtraSettingsAPI_GetCheckboxState(string SettingName) => false;
    public static string ExtraSettingsAPI_GetInputValue(string SettingName) => "";
    public static void ExtraSettingsAPI_SetCheckboxState(string SettingName, bool value) { }
    public static void ExtraSettingsAPI_SetInputValue(string SettingName, string value)  { }
}

[HarmonyPatch(typeof(SO_ObjectSpawner), "GetSettings")]
public class Patch_GetObjectSpawnerSettings
{
    static void Postfix()
    {
        MoreTrashRedux.scheduleCheck = true;
    }
}